@@ -162,13 +162,18 @@ struct RsvgHandlePrivate {
     GString *metadata;
 
     gchar *base_uri;
+    GFile *base_gfile;
 
     gboolean finished;
 
-    gboolean first_write;
-    gboolean is_gzipped;
-    void *gzipped_data;         /* really a GsfOutput */
     gboolean in_loop;		/* see get_dimension() */
+
+    gboolean first_write;
+#if GLIB_CHECK_VERSION (2, 24, 0)
+    GInputStream *data_input_stream; /* for rsvg_handle_write of svgz data */
+#elif defined(HAVE_GSF)
+    void *gzipped_data;         /* really a GsfOutput */
+#endif
 };
 
 typedef struct {
@@ -180,7 +185,7 @@ typedef struct {
 
 struct RsvgDrawingCtx {
     RsvgRender *render;
-    GSList *state;
+    RsvgState *state;
     GError **error;
     RsvgDefs *defs;
     gchar *base_uri;
@@ -250,16 +255,74 @@ struct RsvgSizeCallbackData {
 
 void _rsvg_size_callback (int *width, int *height, gpointer data);
 
+typedef enum {
+    RSVG_NODE_TYPE_INVALID = 0,
+
+    RSVG_NODE_TYPE_CHARS,
+    RSVG_NODE_TYPE_CIRCLE,
+    RSVG_NODE_TYPE_CLIP_PATH,
+    RSVG_NODE_TYPE_COMPONENT_TRANFER_FUNCTION,
+    RSVG_NODE_TYPE_DEFS,
+    RSVG_NODE_TYPE_ELLIPSE,
+    RSVG_NODE_TYPE_FILTER,
+    RSVG_NODE_TYPE_GROUP,
+    RSVG_NODE_TYPE_IMAGE,
+    RSVG_NODE_TYPE_LIGHT_SOURCE,
+    RSVG_NODE_TYPE_LINE,
+    RSVG_NODE_TYPE_LINEAR_GRADIENT,
+    RSVG_NODE_TYPE_MARKER,
+    RSVG_NODE_TYPE_MASK,
+    RSVG_NODE_TYPE_PATH,
+    RSVG_NODE_TYPE_PATTERN,
+    RSVG_NODE_TYPE_POLYGON,
+    RSVG_NODE_TYPE_POLYLINE,
+    RSVG_NODE_TYPE_RADIAL_GRADIENT,
+    RSVG_NODE_TYPE_RECT,
+    RSVG_NODE_TYPE_STOP,
+    RSVG_NODE_TYPE_SVG,
+    RSVG_NODE_TYPE_SWITCH,
+    RSVG_NODE_TYPE_SYMBOL,
+    RSVG_NODE_TYPE_TEXT,
+    RSVG_NODE_TYPE_TREF,
+    RSVG_NODE_TYPE_TSPAN,
+    RSVG_NODE_TYPE_USE,
+
+    /* Filter primitives */
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE = 64,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_BLEND,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_COLOUR_MATRIX,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_COMPONENT_TRANSFER,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_COMPOSITE,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_CONVOLVE_MATRIX,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_DIFFUSE_LIGHTING,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_DISPLACEMENT_MAP,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_ERODE,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_FLOOD,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_GAUSSIAN_BLUR,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_IMAGE,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_MERGE,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_MERGE_NODE,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_OFFSET,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_SPECULAR_LIGHTING,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_TILE,
+    RSVG_NODE_TYPE_FILTER_PRIMITIVE_TURBULENCE,
+
+} RsvgNodeType;
+
 struct _RsvgNode {
     RsvgState *state;
     RsvgNode *parent;
-    GString *type;
     GPtrArray *children;
+    RsvgNodeType type;
+    const char *name; /* owned by the xmlContext, invalid after parsing! */
     void (*free) (RsvgNode * self);
     void (*draw) (RsvgNode * self, RsvgDrawingCtx * ctx, int dominate);
     void (*set_atts) (RsvgNode * self, RsvgHandle * ctx, RsvgPropertyBag *);
 };
 
+#define RSVG_NODE_TYPE(node)                ((node)->type)
+#define RSVG_NODE_IS_FILTER_PRIMITIVE(node) (RSVG_NODE_TYPE((node)) & RSVG_NODE_TYPE_FILTER_PRIMITIVE)
+
 struct _RsvgNodeChars {
     RsvgNode super;
     GString *contents;
@@ -270,7 +333,7 @@ typedef void (*RsvgPropertyBagEnumFunc) (const char *key, const char *value, gpo
 RsvgPropertyBag	    *rsvg_property_bag_new       (const char **atts);
 RsvgPropertyBag	    *rsvg_property_bag_ref       (RsvgPropertyBag * bag);
 void                 rsvg_property_bag_free      (RsvgPropertyBag * bag);
-G_CONST_RETURN char *rsvg_property_bag_lookup    (RsvgPropertyBag * bag, const char *key);
+const char          *rsvg_property_bag_lookup    (RsvgPropertyBag * bag, const char *key);
 guint                rsvg_property_bag_size	     (RsvgPropertyBag * bag);
 void                 rsvg_property_bag_enumerate (RsvgPropertyBag * bag, RsvgPropertyBagEnumFunc func,
                                                   gpointer user_data);
@@ -344,6 +407,7 @@ void rsvg_bbox_clip     (RsvgBbox * dst, RsvgBbox * src);
 double _rsvg_css_normalize_length       (const RsvgLength * in, RsvgDrawingCtx * ctx, char dir);
 double _rsvg_css_hand_normalize_length  (const RsvgLength * in, gdouble pixels_per_inch,
                                          gdouble width_or_height, gdouble font_size);
+double _rsvg_css_normalize_font_size    (RsvgState * state, RsvgDrawingCtx * ctx);
 
 RsvgLength _rsvg_css_parse_length (const char *str);
 
